/**
 * Alipay.com Inc.
 * Copyright (c) 2004-2018 All Rights Reserved.
 */
package com.alipay.fc.customer.cschat.openapiisv.controller;

import com.alibaba.fastjson.JSONObject;
import com.alipay.fc.customer.cschat.openapiisv.service.EncryptUtil;
import com.alipay.fc.customer.cschat.openapiisv.service.MessageHttpClient;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.ArrayUtils;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

/**
 * @author fushan.cfs
 * @version $Id: Index.java, v 0.1 2018年05月24日 上午11:13 fushan.cfs Exp $
 */
public class Index {

    /* 根据自己所在租户的环境选择对应的URL */
    /** 阿里云云客服URL */
    private static final String HOST_URL                 = "https://cschat-ccs.aliyun.com/openapi/";
    /** 金融云云客服URL */
    private static final String HOST_URL_ALIPAY          = "https://cschat.cloud.alipay.com/openapi/";

    //--------------------------------------------------------------------------------

    /* 以下内容需要联系技术支持获取 */
    /** 租户的密钥, 用来计算digest */
    private static final String key              = "HwuF6SWTvNs3NzRDr5WGVX1lrzkCL5t4";

    //--------------------------------------------------------------------------------

    /* 以下内容在开通租户后, 按照文档即可获取, 不懂联系技术支持 */
    /** 租户ID */
    private static final String tntInstId        = "_0GBRgxU";
    /** */
    private static final String scene            = "SCE00000235";

    //--------------------------------------------------------------------------------
    /* 以下内容为固定写死内容, 无需关心 */
    /** 开放渠道, 固定为outerservice */
    private static final String src           = "outerservice";
    /** 发送消息的URL PATH */
    private static final String MESSAGE_PATH             = "forwardMessage";
    /** 上传文件的URL PATH */
    public static final String UPLOAD_FILE_API_PATH     = "/uploadFile";


    public static void main(String[] args) {

        //测试发送文本消息
        //try {
        //    JSONObject res = sendMessage("alps-chen-userId", "你好啊");
        //    System.out.println(res);
        //} catch (Exception e) {
        //    e.printStackTrace();
        //}


        //验证加密
        //System.out.println(EncryptUtil.hmac_sha1("abcd", key));

        //上传二进制文件
        //File file = new File("/file/path");
        //try {
        //    JSONObject res = uploadFile(file, "image", "t.png");
        //    System.out.println(res);
        //    String fileKey = res.getString("fileKey");
        //} catch (IOException e) {
        //    e.printStackTrace();
        //}

        //上传base64的文件, 网页上使用base64文件较多, 可以直接使用这个接口上传
        //String base64File = "";
        //try {
        //    JSONObject res = uploadBase64File(base64File, "image", "t.png");
        //    System.out.println(res);
        //    String fileKey = res.getString("fileKey");
        //} catch (IOException e) {
        //    e.printStackTrace();
        //}

        //发送文件消息
        //try {
        //    JSONObject res = sendMessage("alps-chen-userId", fileKey);
        //    System.out.println(res);
        //} catch (Exception e) {
        //    e.printStackTrace();
        //}

        //获取文件的临时访问链接
        //String fileKey = "";
        //try {
        //    JSONObject res = fetchFileUrl(fileKey);
        //    System.out.println(res);
        //} catch (IOException e) {
        //    e.printStackTrace();
        //}
    }


    /**
     * 发送普通消息接口
     * 通过该接口可以发送: 文本, 图片, 文件
     * @param userId 用户的ID. 用于客户识别该用户
     * @param content 消息的内容, 文本消息是文本内容, 图片/文件类型内容是fileKey
     * @return 返回Http send结果
     * @throws Exception
     */
    public static JSONObject sendMessage(String userId, String content) throws Exception {
        //拿到当前的 毫秒 时间戳
        long timestamp = System.currentTimeMillis();

        //构造一个文本消息 消息的字段参考文档
        JSONObject message = new JSONObject();
        message.put("userId", userId);
        message.put("msgType", "text");
        message.put("content", content);
        message.put("timestamp", timestamp);
        //messageStr 就是发送请求的post body内容, 也是计算摘要的内容. 一定要保持一致, 否则digest计算失败
        String messageStr = message.toJSONString();

        //计算安全摘要, 计算的传入内容请参考文档
        //如果想要知道自己计算的摘要是否正确
        //校验网址: https://1024tools.com/hmac
        //选择算法: sha1
        //密钥填写: key的内容
        //内容就是hmac_sha1的value内容
        String digest = EncryptUtil.hmac_sha1(messageStr + timestamp, key);

        //拼接URL
        String messageUrl = HOST_URL  + MESSAGE_PATH + "?tntInstId=" + tntInstId + "&scene=" + scene + "&digest=" + digest
                + "&timestamp=" + timestamp + "&src=" + src;

        //发送消息
        //成功返回内容为:
        //{
        //    "code": "200",
        //    "msg": "success"
        //}
        //失败情况下, 请查看返回code, 参考文档
        MessageHttpClient httpClient = new MessageHttpClient();
        return httpClient.sendMessage(messageStr, messageUrl);
    }

    /**
     * 上传二进制文件
     * 上传文件后返回里会拿到fileKey
     * fileKey通过发送消息(sendMessage)来发送到客服
     * @param file 文件
     * @param type image/voice/video
     * @param fileName 文件名字,保证后缀名正确即可
     * @return 上传结果
     * @throws IOException 抛出异常
     */
    public static JSONObject uploadFile(File file, String type, String fileName) throws IOException {
        //拿到当前的 毫秒 时间戳
        long timestamp = System.currentTimeMillis();

        byte[] fileBytes = IOUtils.toByteArray(new FileInputStream(file));
        byte[] timeBytes = Long.toString(timestamp).getBytes();
        byte[] digestBytes = ArrayUtils.addAll(fileBytes, timeBytes);
        String digest = EncryptUtil.hmac_sha1(digestBytes, key);

        //拼接URL
        String fileUrl = HOST_URL  + UPLOAD_FILE_API_PATH + "?tntInstId=" + tntInstId + "&scene=" + scene + "&digest=" + digest
                + "&timestamp=" + timestamp + "&src=" + src;

        MessageHttpClient httpClient = new MessageHttpClient();
        return httpClient.uploadFile(file, fileUrl, type, fileName, timestamp);
    }

    /**
     * 上传base64文件
     * @param base64File base64文件内容
     * @param type image/voice/video
     * @param fileName 文件名字,保证后缀名正确即可
     * @return 上传结果
     * @throws IOException 抛出异常
     */
    public static JSONObject uploadBase64File(String base64File, String type, String fileName) throws IOException {
        //拿到当前的 毫秒 时间戳
        long timestamp = System.currentTimeMillis();

        //计算摘要
        String digest = EncryptUtil.hmac_sha1(base64File + timestamp, key);

        //拼接URL
        String fileUrl = HOST_URL  + UPLOAD_FILE_API_PATH + "?tntInstId=" + tntInstId + "&scene=" + scene + "&digest=" + digest
                + "&timestamp=" + timestamp + "&src=" + src;

        MessageHttpClient httpClient = new MessageHttpClient();
        return httpClient.uploadBase64File(base64File, fileUrl, type, fileName, timestamp);
    }


    /**
     * 获取文件的临时url
     * @param fileKey 文件的file key (在上传的时候获取，或者在接收消息的时候由云客服发送过来的)
     * @return
     * success 返回内容(json)：
     * <ul>
     *     <li>type : image/video/voice</li> 文件类型
     *     <li>mediaUrl: "https://oss.aliyun.com/example/url</li> 文件临时url
     *     <li>expiretime : 12344312</li> url过期时间
     * </ul>
     * error 返回内容：
     * <ul>
     *     <li>code : 404</li> 错误码
     *     <li>msg : "error message"</li> 错误消息提示
     * </ul>
     * @throws IOException 异常内容 一般情况下都是httpClient抛出的
     */
    public static  JSONObject fetchFileUrl(String fileKey) throws IOException {
        //获取时间戳
        long timestamp = System.currentTimeMillis();
        String digest = EncryptUtil.hmac_sha1(fileKey + timestamp, key);

        //拼接URL
        String fileUrl = HOST_URL  + UPLOAD_FILE_API_PATH + "?fileKey=" + fileKey + "&timestamp=" + timestamp + "&digest=" + digest
                + "&timestamp=" + timestamp + "&src=" + src;

        MessageHttpClient httpClient = new MessageHttpClient();
        return httpClient.fetchFileUrl(fileUrl);
    }
}